<?php

require_once 'main_helper.php';
require_once 'lib/abstract_epfs_handler.php';

class EpfsHandler extends AbstractEpfsHandler
{
    const DATA_TS = '/config/globaltv_data_timestamp';
    const CFG_TS = '/config/globaltv_cfg_timestamp';

    private static $no_internet_epfs = array(
      'tv',
    );

    private static $no_internet_epfs_created = false;

    private static function get_all_section_ids()
    {
        MainHelper::$tv->ensure_channels_loaded();
        $section_by_id = MainHelper::$tv->get_sections();
        $ids = array_keys($section_by_id);
        if (MainHelper::with_favorites())
            array_unshift($ids, SectionsHelper::FAV_SECTION_ID);
        return $ids;
    }

    public static function update_fav_epfs()
    {
        $plugin_cookies = MainHelper::$plugin->get_plugin_cookies();

        $cache = MainHelper::$cache;
        $cache->ensure_cache_loaded();

        $section_id = SectionsHelper::FAV_SECTION_ID;

        $ts1 = microtime(true);

        try
        {
            $folder_view = MainHelper::$tv_rows_screen->
                get_folder_view_for_epf($section_id, $plugin_cookies);
        }
        catch (Exception $e)
        {
            hd_print("Exception while generating epf: ".$e->getMessage());
            return;
        }

        $epf_id = self::get_epf_id($section_id);
        self::write_epf($epf_id, $folder_view);

        $ts2 = microtime(true);
        hd_print(sprintf("fav epf generated in %.2fs", $ts2 - $ts1));

        $doc = self::make_rows_menu_config();
        self::write_rmc($doc);

        // TODO: revise
        $ts = self::make_monotonic_ts();
        self::write_epfs_ts($ts, 'fav');
    }

    private static function ensure_no_internet_epfs_created($first_run)
    {
        if (self::$no_internet_epfs_created)
            return;

        foreach (self::$no_internet_epfs as $epf_id)
        {
            $path = self::get_no_internet_epf_path($epf_id);
            if (!$first_run && is_file($path))
                continue;

            $folder_view = MainHelper::$dummy_epf_screen->
                get_folder_view_for_epf(true, $plugin_cookies);
            self::write_no_internet_epf($epf_id, $folder_view);
        }

        self::$no_internet_epfs_created = true;
    }

    private static function make_tag()
    {
        $cfg = SystemConfig::get_instance();
        $cfg->reload();
        return "$cfg->lang_code-$cfg->country_code_uc";
    }

    public static function update_all_epfs($first_run)
    {
        clearstatcache();

        self::ensure_no_internet_epfs_created($first_run);

        // Check changes for epf modules.

        $cold_run = !is_file(self::warmed_up_path());

        $cached_ts = self::read_epfs_ts('tv');

        $data_ts = self::read_data_ts(self::DATA_TS);
        $cfg_ts = self::read_data_ts(self::CFG_TS);
        $src_ts = self::max_ts($data_ts, $cfg_ts);

        $data_changed = !$cached_ts ||
            self::cmp_ts($cached_ts, $data_ts) < 0;
        if ($data_changed)
            hd_print("epfs D: data changed ($cached_ts < $data_ts)");

        $changed = 0;

        // TODO: revise cold run
        if ($cold_run) {
            $changed = 1;
            hd_print("epfs D: cold run");
        } else if (!$cached_ts) {
            $changed = 1;
            hd_print("epfs D: not cached yet");
        } else if (self::cmp_ts($cached_ts, $src_ts) < 0) {
            $changed = 1;
            hd_print("epfs D: cache outdated ($cached_ts < $src_ts)");
        }

        $tag = self::make_tag();
        $last_tag = self::read_tag();
        if ($tag != $last_tag) {
            hd_print("epfs D: tag changed $last_tag -> $tag");
            $changed = 1;
        }

        if (!$changed)
        {
            hd_print("update_all_epfs D: not changed");
            return null;
        }
        hd_print("update_all_epfs I: changed, updating");

        ///////////////////////////////////////////////////////////////////
        
        $plugin_cookies = MainHelper::$plugin->get_plugin_cookies();

        $cache = MainHelper::$cache;
        $tv = MainHelper::$tv;

        if ($data_changed)
        {
            $cache->clear_all();
            $tv->unload_channels();
        }

        $cache_ok = false;
        try
        {
            if ($cache->ensure_cache_loaded())
            {
                $tv->ensure_channels_loaded();
                $cache_ok = true;
            }
        }
        catch (Exception $e)
        {
            hd_print("Exception while generating epf: ".$e->getMessage());
        }

        if ($cache_ok)
        {
            $enabled_idset = SectionsHelper::get_enabled_sections_idset(true);
            $section_ids = self::get_all_section_ids();

            $n = count($enabled_idset) +
                (MainHelper::with_favorites()?1:0);
            if ($n > 0)
                hd_print("Updating $n section epfs");

            foreach ($section_ids as $section_id)
            {
                if ($section_id != SectionsHelper::FAV_SECTION_ID &&
                    !isset($enabled_idset[$section_id]))
                {
                    continue;
                }

                $ts1 = microtime(true);

                $folder_view = MainHelper::$tv_rows_screen->
                    get_folder_view_for_epf($section_id, $plugin_cookies);

                $epf_id = self::get_epf_id($section_id);
                self::write_epf($epf_id, $folder_view);

                $ts2 = microtime(true);
                hd_print(sprintf("TV epf generated in %.2fs", $ts2 - $ts1));
            }

            $doc = self::make_rows_menu_config();
            self::write_rmc($doc);
        }
        else
        {
            $no_internet = MainHelper::without_internet();
            $folder_view = MainHelper::$dummy_epf_screen->
                get_folder_view_for_epf($no_internet, $plugin_cookies);
            self::write_epf('products', $folder_view);
        }

        // Write epfs timestamps for changed modules.

        if ($tag != $last_tag)
            self::write_tag($tag);

        self::write_epfs_ts($src_ts, 'tv');
        self::write_epfs_ts($src_ts, 'fav');

        if ($cold_run)
            file_put_contents(self::warmed_up_path(), "");

        return null;
    }

    public static function force_update_section($section_id)
    {
        $plugin_cookies = MainHelper::$plugin->get_plugin_cookies();
        $is_fav = $section_id == SectionsHelper::FAV_SECTION_ID;

        $cache = MainHelper::$cache;
        $tv = MainHelper::$tv;

        $cache_ok = false;
        try
        {
            if ($cache->ensure_cache_loaded())
            {
                $tv->ensure_channels_loaded();
                $cache_ok = true;
            }
        }
        catch (Exception $e)
        {
            hd_print("Exception while generating epf: ".$e->getMessage());
        }
        if (!$cache_ok)
        {
            hd_print("epfs E: failed to force-update $section_id");
            return;
        }

        $ts1 = microtime(true);

        $folder_view = MainHelper::$tv_rows_screen->
            get_folder_view_for_epf($section_id, $plugin_cookies);

        $epf_id = self::get_epf_id($section_id);
        self::write_epf($epf_id, $folder_view);

        $ts2 = microtime(true);
        hd_print(sprintf("TV epf ($section_id) generated in %.2fs", $ts2 - $ts1));

        if ($is_fav)
        {
            // TODO: revise
            $ts = self::make_monotonic_ts();
            self::write_epfs_ts($ts, 'fav');
        }
    }

    private static function make_rows_menu_config()
    {
        $cache = MainHelper::$cache;
        $cache->ensure_cache_loaded();

        $pname = self::get_plugin_name();

        $items = array();

        if (MainHelper::with_favorites())
        {
            $epf_id = self::get_epf_id(SectionsHelper::FAV_SECTION_ID);
            $items[] = array(
                'type' => 'plugin',
                'name' => $pname,
                'epf_id' => $epf_id,
                'caption' => T::todo('MY'),
                "small_icon_url" => "plugin_file://%$pname%/icons/my.png",
                'left' => 0,
            );
        }

        $enabled_idset = SectionsHelper::get_enabled_sections_idset(true);

        $ndx = 0;
        MainHelper::$tv->ensure_channels_loaded();
        $sections = MainHelper::$tv->get_sections();
        foreach ($sections as $id => $section)
        {
            if (!isset($enabled_idset[$id]))
                continue;

            $small_icon_url = SectionsHelper::section_get_icon_url($section);

            $badge_ndx = $ndx + 5;
            $epf_id = self::get_epf_id($section['id']);
            $arr = array(
                'type' => 'plugin',
                'name' => $pname,
                'epf_id' => $epf_id,
                'caption' => $section['name'],
//                'icon_url' => "plugin_file://%shell_ext%/icons/badge$badge_ndx.png",
                "small_icon_url" => $small_icon_url,
                'left' => 5,
            );
            if ($id == 'xxx')
                $arr['protected'] = 1;
            $items[] = $arr;

            $ndx = ($ndx + 1) % 10;
        }

        return array('rows_menu_items' => $items);
    }

    public static function get_epf_id($section_id)
    {
        if ($section_id == SectionsHelper::FAV_SECTION_ID)
            return 'section_fav';
        return "section_$section_id";
    }
}

?>
