<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/abstract_preloaded_regular_screen.php';

///////////////////////////////////////////////////////////////////////////

class GlobalTvTvCategoryListScreen extends AbstractPreloadedRegularScreen
    implements UserInputHandler
{
    const ID = 'tv_category_list';

    ///////////////////////////////////////////////////////////////////////

    public static function get_media_url_str($section_id)
    {
        return MediaURL::encode(array(
            'screen_id'  => self::ID,
            'section_id' => $section_id,
        ));
    }

    protected $tv;

    ///////////////////////////////////////////////////////////////////////

    public function __construct($tv, $folder_views)
    {
        parent::__construct(self::ID, $folder_views);
        $this->tv = $tv;
        UserInputHandlerRegistry::get_instance()->register_handler($this);
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_handler_id()
    { return self::ID; }

    public function get_sel_item_update_action(&$user_input, &$plugin_cookies)
    {
        $media_url = MediaURL::decode($user_input->selected_media_url);
        $sel_ndx = $user_input->sel_ndx;
        $section_id = $media_url->section_id;
        $group_ids_in_section = $this->tv->get_group_ids_by_section_id($section_id, $plugin_cookies);
        if ($section_id == SectionsHelper::FAV_SECTION_ID)
        {
            $parent_media_url = MediaURL::decode($user_input->parent_media_url);

            $num_favorites = count($group_ids_in_section) + 1;
            if ($sel_ndx >= $num_favorites)
                $sel_ndx = $num_favorites - 1;

            $range = HD::create_regular_folder_range(
                $this->get_all_folder_items($parent_media_url, $plugin_cookies));
            return ActionFactory::update_regular_folder(
                $range, true, $sel_ndx);
        }

        $gid = $media_url->group_id;
        $group_by_id = $this->tv->get_groups();
        $g = HD::get($group_by_id, $gid);
        $starred = $this->tv->is_favorite_category_id($gid, $plugin_cookies);

        $items = array();
        $this->add_group_item($items,
            GlobalTvTvChannelListScreen::get_media_url_str($g['id'], $section_id),
            $g['name'], HD::get($g, 'panel_icon_url', ''), $starred);

        $range = HD::create_regular_folder_range($items, $sel_ndx, count($group_ids_in_section));
        return ActionFactory::update_regular_folder($range, false);
    }

    public function get_action_map(MediaURL $media_url, &$plugin_cookies)
    {
        $open_action =
            UserInputHandlerRegistry::create_action(
                $this, 'folder_open');
        $play_action =
            UserInputHandlerRegistry::create_action(
                $this, 'play_tv');

        $actions = array();
        $actions[GUI_EVENT_KEY_ENTER] = $open_action;//ActionFactory::open_folder();
        $actions[GUI_EVENT_KEY_PLAY] = $play_action;//ActionFactory::tv_play();

        if ($this->tv->is_favorites_supported())
        {
            $add_favorite_action =
                UserInputHandlerRegistry::create_action(
                    $this, 'toggle_favorite');
            $add_favorite_action['caption'] = T::g('plugin_default_favorites');

            $popup_menu_action =
                UserInputHandlerRegistry::create_action(
                    $this, 'popup_menu');

            $actions[GUI_EVENT_KEY_D_BLUE] = $add_favorite_action;
            $actions[GUI_EVENT_KEY_POPUP_MENU] = $popup_menu_action;
        }

        $actions[GUI_EVENT_KEY_INFO] =
            UserInputHandlerRegistry::create_action(
                $this, 'info');

        return $actions;
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_all_folder_items(MediaURL $media_url, &$plugin_cookies)
    {
        $this->tv->ensure_channels_loaded();
        $items = array();

        $section_id = $media_url->section_id;
        $is_fav = $section_id == SectionsHelper::FAV_SECTION_ID;
        hd_print("TV category list: get_all_folder_items, section_id: $section_id");

        if ($is_fav)
        {
            $g = $this->tv->get_favorites_group();
            $this->add_group_item($items,
                TvFavoritesScreen::get_media_url_str(),
                $g['name'], $g['icon_url'], false);
        }

        $group_ids_in_section = $this->tv->get_group_ids_by_section_id($section_id, $plugin_cookies);
        $group_by_id = $this->tv->get_groups();
        foreach ($group_ids_in_section as $gid)
        {
            $g = HD::get($group_by_id, $gid);
            $starred = !$is_fav && $this->tv->is_favorite_category_id($gid, $plugin_cookies);
            $this->add_group_item($items,
                GlobalTvTvChannelListScreen::get_media_url_str($g['id'], $section_id),
                $g['name'], HD::get($g, 'panel_icon_url', ''), $starred);
        }

        return $items;
    }

    private function add_group_item(&$items, $media_url, $name, $icon_url, $starred)
    {
        $items[] = array(
            PluginRegularFolderItem::media_url => $media_url,
            PluginRegularFolderItem::caption => $name,
            PluginRegularFolderItem::view_item_params => array(
                ViewItemParams::icon_path => $icon_url,
                ViewItemParams::item_detailed_icon_path => $icon_url,
            ),
            PluginRegularFolderItem::starred => $starred,
        );
    }

    public function get_archive(MediaURL $media_url)
    {
        return $this->tv->get_archive($media_url);
    }

    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        hd_print('TV category list: handle_user_input:');
        foreach ($user_input as $key => $value)
            hd_print("  $key => $value");

        $group_id = null;
        if (isset($user_input->selected_media_url))
        {
            $media_url = MediaURL::decode($user_input->selected_media_url);
            $group_id = $media_url->group_id;
        }

        $control_id = HD::get($user_input, 'control_id');
        if ($control_id == 'info')
        {
            if (!$group_id)
                return null;

            $group = $this->tv->get_group_by_id($group_id);
            $id = $group["id"];
            $title = $group["name"];

            $msg = "TV group '$title' (id=$id)";
            hd_print($msg);
            return ActionFactory::show_title_dialog($msg);
        }
        else if ($control_id == 'popup_menu')
        {
            if (!$group_id)
                return null;

            $is_favorite = $this->tv->is_favorite_category_id(
                $group_id, $plugin_cookies);
            $caption = $is_favorite ?
                T::g('navigator_popup_remove_from_favorites') :
                T::g('navigator_popup_add_to_favorites');
            $add_favorite_action =
                UserInputHandlerRegistry::create_action(
                    $this, 'toggle_favorite');

            $menu_items = array();
            $menu_items[] = array(
                GuiMenuItemDef::caption => $caption,
                GuiMenuItemDef::action => $add_favorite_action);

            return ActionFactory::show_popup_menu($menu_items);
        }
        else if ($control_id == 'toggle_favorite')
        {
            if (!$group_id)
                return null;

            $is_favorite = $this->tv->is_favorite_category_id(
                $group_id, $plugin_cookies);
            $text = "";

            if ($is_favorite)
            {
                $this->tv->change_tv_favorites_categories(PLUGIN_FAVORITES_OP_REMOVE,
                    $group_id, $plugin_cookies);
                $text = T::t('category_removed_from_favorites');
            }
            else
            {
                $this->tv->change_tv_favorites_categories(PLUGIN_FAVORITES_OP_ADD,
                    $group_id, $plugin_cookies);
                $text = T::t('category_added_to_favorites');
            }

            $update_action = $this->get_sel_item_update_action($user_input, $plugin_cookies);
            $action = ActionFactory::show_title_dialog($text, $update_action);
            if (!HD::with_rows_api())
                return $action;

            return $this->tv->invalidate_favorites($action);
        }

        $action = MainHelper::get_pf_check_action_if_needed($user_input, "TV category list", false);
        if ($action != null)
            return $action;

        $a = MainHelper::get_pre_playback_action_for_group_if_needed($this, $group_id, $user_input, $plugin_cookies);
        if ($a !== true)
            return $a;

        $control_id = HD::get($user_input, 'control_id');
        if ($control_id == 'folder_open')
            return ActionFactory::open_folder();
        else if ($control_id == 'play_tv')
            return ActionFactory::tv_play();

        return null;
    }
}

///////////////////////////////////////////////////////////////////////////
?>
