<?php

require_once 'global_settings.php';
require_once 'lib/user_input_handler_registry.php';

class MainEntryHandler implements UserInputHandler
{
    const INSTALLED_PLUGINS_FILE =
        '/tmp/run/installed_plugins.xml';

    private $session;

    private $last_text = '';

    public function __construct($session)
    {
        $this->session = $session;

        UserInputHandlerRegistry::get_instance()->
            register_handler($this);
    }

    public function get_handler_id()
    {
        return 'entry';
    }

    public function custom_action($control_id, $params=null)
    {
        return UserInputHandlerRegistry::create_action($this,
            $control_id, $params);
    }

    private function use_rows_screen($entry_id)
    {
        if (!HD::with_rows_api())
            return false;

        if ($entry_id == 'top' && GlobalSettings::USE_ROWS_SCREEN_TOP)
            return true;

        return false;
    }

    private function get_topmenu_popup_action($item_id, $pcode_entered)
    {
        $pname = MainHelper::$plugin_name;
        if (0 !== strpos($item_id, "plugin:$pname:"))
            return null;

        $name = substr($item_id, strlen("plugin:$pname:"));
        if (substr($name, 0, 8) != 'section_')
            return null;
        $section_id = substr($name, 8);

        $protected = $section_id == 'xxx' && !$pcode_entered &&
            MainHelper::read_xxx_password() != '';
        if ($protected)
        {
            if (HD::with_builtin_pcontrol()) {
                $action = ActionFactory::show_error(false,
                    T::g('msg_restricted_by_parental_code'));
            } else {
                return null;
            }
        } else {
            $eff_section_id = $section_id == 'xxx' ? 'xxx' : 'all';
            $action = $this->custom_action('search_channels',
                array('section_id' => $eff_section_id));
        }
        $menu_items[] = ActionFactory::get_menu_item(
            T::t("popup_action_search_tv_channels"),
            $action,
            "plugin_file://%shell_ext%/icons/search_small.png");

        $action = $this->custom_action('configure_sections');
        $menu_items[] = ActionFactory::get_menu_item(
            T::t("popup_action_configure_sections"),
            $action,
            "plugin_file://%shell_ext%/icons/setup_small.png");
        
        if (!$protected && HD::with_list_config())
        {
            $caption = T::t("popup_action_configure_categories");
            $a = MainHelper::$list_helper->edit_cats_list($section_id);
            $menu_items[] = ActionFactory::get_menu_item(
                $caption, $a, null);
        }

        return ActionFactory::add_menu_items($menu_items);
    }

    private function show_search_channels_dialog($section_id)
    {
        $pc = MainHelper::$plugin->get_plugin_cookies();
        $text = HD::get($pc, 'last_search_channels_text', '');

        $defs = array();

        ControlFactory::add_label($defs, null, 
            T::t('label_search_string'));
        ControlFactory::add_vgap($defs, -5);
        ControlFactory::add_text_field($defs, $this, null,
            'search_channels_text', null, $text,
            false, false, false, true, 800);

        ControlFactory::add_vgap($defs, 20);

        $bparams = array('button_caption_centered' => true);
        $add_params = array('section_id' => $section_id);
        ControlFactory::add_button($defs, $this, $add_params,
            'search_channels_submit_dialog', null, T::key_global('button_ok'),
            300, $bparams);
        ControlFactory::add_close_dialog_button($defs,
            T::key_global('button_cancel'), 300, $bparams);

        return ActionFactory::show_dialog(
            T::t('title_search_tv_channels'), $defs, true, 800);
    }

    private function search_channels_submit($text, $section_id)
    {
        if ($section_id != 'xxx')
        {
            $pc = MainHelper::$plugin->get_plugin_cookies();
            $pc->last_search_channels_text = $text;
        }

        $media_url = TvRowsScreen::get_media_url_str(
            'search', $section_id, $text);
        $caption = T::t1('path_item_search__1', $text);
        hd_print("search_channels_submit: text: '$text', section_id: '$section_id', media_url: '$media_url'");
        return ActionFactory::open_folder($media_url, $caption);
    }

    private function configure_sections()
    {
        MainHelper::$cache->ensure_cache_loaded();

        $title = T::t('title_sections_setup');
        $apply_action = $this->custom_action('apply_sections');

        $cache = MainHelper::$cache;
        $section_by_id = $cache->get_section_by_id_map();

        $all_by_id = array();
        foreach ($section_by_id as $id => $section)
        {
            $caption = $section['name'];
            $icon_url = SectionsHelper::section_get_icon_url($section);
            $all_by_id[$id] = (object) array(
                'id' => $id,
                'caption' => $caption,
                'icon_url' => $icon_url);
        }

        $checked_idset = SectionsHelper::get_enabled_sections_idset(false);

        $main_ids = $cache->get_main_section_ids();
        $other_ids = $cache->get_other_section_ids($main_ids);
        $group_by_id = array(
            'default' => (object) array(
                'caption' => T::t('item_default_sections'),
                'ids' => $main_ids,
            ),
            'other' => (object) array(
                'caption' => T::t('item_other_sections'),
                'ids' => $other_ids,
            ));

        return MainHelper::$edit_list_dialog->show($title, 870,
            $all_by_id, $checked_idset, $group_by_id, $apply_action);
    }

    private function apply_sections($user_input)
    {
        $checked_idset =
            json_decode($user_input->checked_idset, true);
        $initial_checked_idset =
            json_decode($user_input->initial_checked_idset, true);

        $path = SectionsHelper::get_sections_on_off_path();
        $on_off = ListUtils::read_on_off_file($path);
        ListUtils::update_on_off($on_off,
            $initial_checked_idset, $checked_idset);

        $st = ListUtils::write_on_off_file($path, $on_off);
        if ($st != 1) {
            hd_print("Sections configuration not changed");
            return null;
        }

        EpfsHandler::write_data_ts(EpfsHandler::CFG_TS);

        EpfsHandler::update_all_epfs(false);
        return ActionFactory::reread_all_epfs(true);
    }

    private function autoresume(&$user_input, &$plugin_cookies)
    {
        $gid = HD::get($user_input, 'resume_tv_group');
        $ch_id = HD::get($user_input, 'resume_tv_channel');
        $is_fav = HD::get($user_input, 'resume_tv_is_favorite', 0);
        $murl_str = HD::get($user_input, 'resume_media_url');

        if (!isset($gid))
        {
            hd_print("autoresume E: resume_tv_group missing");
            return null;
        }
        if (!isset($ch_id))
        {
            hd_print("autoresume E: resume_tv_channel missing");
            return null;
        }
        if (!isset($murl_str))
        {
            hd_print("autoresume E: resume_media_url missing");
            return null;
        }

        if (MainHelper::$tv->channel_is_protected($ch_id))
        {
            hd_print("autoresume I: resume XXX suppressed");
            return null;
        }

        $a = MainHelper::get_pre_playback_action_if_needed($this, $ch_id, $user_input, $plugin_cookies);
        if ($a !== true)
            return $a;

        $murl = MediaURL::decode($murl_str);
        $resume_murl = MediaURL::encode(array(
            'section_id' => $murl->section_id,
            'group_id' => $gid,
            'channel_id' => $ch_id,
        ));

        hd_print("Resume playback");
        return ActionFactory::tv_play($resume_murl);
    }

    private function load_installed_plugins()
    {
        $installed_plugins = array();

        $doc = file_get_contents(self::INSTALLED_PLUGINS_FILE);
        if (!$doc)
        {
            hd_print("Empty or missing installed plugins file");
            return $installed_plugins;
        }
        $xml = simplexml_load_string($doc);
        if (!$xml)
        {
            hd_print("Failed to parse installed plugins file");
            return $installed_plugins;
        }

        foreach ($xml->children() as $plugin)
        {
            $name = strval($plugin->name);
            $installed_plugins[$name] = (object) array(
                'name' => $name);
        }

        return $installed_plugins;
    }

    private function country_is_cn()
    {
        $cfg = SystemConfig::get_instance();
        $cfg->ensure_loaded();

        $val = $cfg->widget_country ?
            $cfg->widget_country : $cfg->country_code_uc;
        $val = mb_strtoupper($val, 'UTF-8');
        hd_print("Effective country: $val");
        return in_array($val, array('CN', 'CHINA'));
    }

    private function do_handle_user_input(&$user_input, &$plugin_cookies)
    {
        hd_print('entry handler D: handle_user_input:');
        foreach ($user_input as $key => $value)
            hd_print("  $key => $value");

        $entry_id = HD::get($user_input, 'entry_id', '');
        $control_id = HD::get($user_input, 'control_id', '');

        if ($entry_id == 'install')
        {
            $dpath = DuneSystem::$properties['install_dir_path'];
            $bin = "$dpath/bin/install.sh";
            hd_print("install I: running $bin");
            system($bin, $ret);
            if ($ret != 0)
                hd_print("install W: cmd exit status: $ret");

            if (!$this->country_is_cn())
                return null;

            $plugins = $this->load_installed_plugins();
            if (!$plugins || !isset($plugins['tv_app_supplier_globaltv']))
                return null;

            $to_uninst = array('tv_app_supplier_globaltv');
            if (isset($plugins['globaltv']))
                $to_uninst[] = 'globaltv';
            $n = count($to_uninst);
            hd_print("install I: going to uninstall $n plugin(s)");

            $action = null;
            foreach (array_reverse($to_uninst) as $plugin_name)
            {
                $url = "plugin_uninstaller://$plugin_name";
                $action = ActionFactory::launch_media_url($url, $action);
            }
            return $action;
        }

        if ($entry_id == 'update_epfs')
        {
            if (!HD::with_rows_api())
                return null; // Should not happen.

            try
            {
                $first_run = HD::get($user_input, 'first_run_after_boot') ||
                    HD::get($user_input, 'restore_from_sleep');
                EpfsHandler::update_all_epfs($first_run);
            }
            catch (Exception $e)
            {
                hd_print("Exception at update_epfs: " . print_r($e, true));
            }
            return ActionFactory::status(0);
        }

        if ($control_id == 'cancel')
            return null;

        if ($this->use_rows_screen($entry_id))
            return ActionFactory::open_folder(TvRowsScreen::ID, 'TV');

        if ($entry_id == 'top')
        {
            MainHelper::$cache->ensure_cache_loaded();
            return ActionFactory::open_folder(GlobalTvTvSectionListScreen::ID, "Global TV");
        }

        if ($entry_id == 'topmenu_popup')
        {
            $item_id = HD::get($user_input, 'selected_topmenu_item_id');
            if (!$item_id)
                return null;
            $pcode_entered = HD::get($user_input, 'topmenu_pcode_entered', 0);
            return $this->get_topmenu_popup_action($item_id, $pcode_entered);
        }

        if ($control_id == 'topmenu_timer')
        {
            clearstatcache();
            HD::safe_unlink(MainHelper::$update_cache_path);
            HD::safe_unlink(MainHelper::$zip_path);

            $a = ActionFactory::action_from_user_input($user_input,
                array('control_id' => 'topmenu_update1'));
            $url = MainHelper::$main_ts_url;
            return ActionFactory::wget($url, null, null,
                'globaltv_update1', null, null, 'update1_', $a);
        }

        if ($control_id == 'topmenu_update1')
        {
            $status = HD::get($user_input, 'update1_status');
            if ($status != 'succeeded')
            {
                hd_print("E: error fetching timestamp from: $user_input->update1_url");
                return ActionFactory::status(1);
            }
            $ts = trim(HD::get($user_input, 'update1_text', ''));
            if (!$ts)
            {
                hd_print("E: empty or missing timestamp from: $user_input->update1_url");
                return ActionFactory::status(1);
            }
            $old_ts = is_file(MainHelper::$main_ts_path) ?
                trim(file_get_contents(MainHelper::$main_ts_path)) : '';
            if ($ts == $old_ts)
            {
                hd_print("D: ts not changed ($ts)");
                return ActionFactory::status(0);
            }

            hd_print("D: ts changed '$old_ts' -> '$ts'");
            file_put_contents(MainHelper::$update_ts_path, $ts);

            $a = ActionFactory::action_from_user_input($user_input,
                array('control_id' => 'topmenu_update2'));
            $url = MainHelper::$main_cache_url;
            $path = MainHelper::$zip_path;
            $dpath = MainHelper::$base_data_path;
            $zname = MainHelper::$zip_name;
            $cmd = "cd $dpath && unzip $zname && rm -f $zname";
            return ActionFactory::wget($url, null, null,
                'globaltv_update2', $path, $cmd, 'update2_', $a);
        }

        if ($control_id == 'topmenu_update2')
        {
            $status = HD::get($user_input, 'update2_status');
            if ($status != 'succeeded')
            {
                hd_print("Error fetching cache from: $user_input->update2_url");
                HD::safe_unlink(MainHelper::$zip_path);
                return ActionFactory::status(1);
            }

            clearstatcache();
            $path = MainHelper::$update_cache_path;
            if (!is_file($path))
            {
                hd_print("File missing unexpectedly: $path");
                HD::safe_unlink(MainHelper::$zip_path);
                return ActionFactory::status(1);
            }

            $from = MainHelper::$update_cache_path;
            $to = MainHelper::$main_cache_path;
            if (!rename($from, $to))
            {
                hd_print("Cache E: rename($from, $to) failed");
                return ActionFactory::status(1);
            }

            $from = MainHelper::$update_ts_path;
            $to = MainHelper::$main_ts_path;
            if (!rename($from, $to))
                hd_print("Cache W: rename($from, $to) failed");

            MainHelper::tv_cache_clear_all();

            EpfsHandler::write_data_ts(EpfsHandler::DATA_TS);
            file_put_contents(EpfsHandler::get_epfs_changed_path(), '');

            $ts = file_get_contents(MainHelper::$main_ts_path);
            hd_print("Cache updated OK, new timestamp: $ts");
            return ActionFactory::status(0);
        }

        if ($control_id == 'search_channels')
        {
            $section_id = HD::get($user_input, 'section_id', 'all');
            return $this->show_search_channels_dialog($section_id);
        }

        if ($control_id == 'search_channels_submit_dialog')
        {
            $text = trim($user_input->search_channels_text);
            if ($text == '')
            {
                return ActionFactory::show_error(false,
                    T::g('dialog_utils_nonempty_value_required'));
            }

            $section_id = HD::get($user_input, 'section_id', 'all');

            $text = mb_strtolower($text, 'UTF-8');
            return ActionFactory::close_dialog_and_run(
                $this->search_channels_submit($text, $section_id));
        }

        if ($control_id == 'configure_sections')
            return $this->configure_sections();

        if ($control_id == 'apply_sections')
            return $this->apply_sections($user_input);

        $debug_name = $entry_id ? $entry_id : $control_id;
        $action = MainHelper::get_pf_check_action_if_needed($user_input, $debug_name, true);
        if ($action != null)
            return $action;

        $pf_playback_available = true;
        if (isset($user_input->pf_error))
            $pf_playback_available = false;
        MainHelper::$cache->set_pf_playback_available($pf_playback_available);

        if ($entry_id == 'autoresume')
            return $this->autoresume($user_input, $plugin_cookies);

        if (!empty($user_input->callback_action)) {
            return json_decode($user_input->callback_action, true);
        }

        return null;
    }

    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        try
        {
            return $this->do_handle_user_input($user_input, $plugin_cookies);
        }
        catch (Exception $e)
        {
            throw $this->session->dune_api_exception(
                $e, $e->getMessage(), false);
        }
    }
}

?>
