<?php

require_once 'lib/default_dune_plugin.php';
require_once 'lib/utils.php';
require_once 'lib/lang.php';
require_once 'lib/list_utils.php';
require_once 'folder_views.php';
require_once 'main_tv.php';
require_once 'globaltv_tv_favorites_screen.php';
require_once 'globaltv_tv_section_list_screen.php';
require_once 'globaltv_tv_category_list_screen.php';
require_once 'globaltv_tv_channel_list_screen.php';
require_once 'session.php';
require_once 'main_entry_handler.php';
require_once 'sections_helper.php';
require_once 'country_helper.php';
require_once 'edit_list_dialog.php';

require_once 'main_helper.php';
require_once 'cache.php';

if (HD::with_rows_api())
{
    require_once 'lib/dummy_epf_screen.php';
    require_once 'tv_rows_screen.php';
    require_once 'epfs_handler.php';
}

if (HD::with_list_config())
{
    require_once 'config_holder.php';
    require_once 'list_helper.php';
}

class MainPlugin extends DefaultDunePlugin
{
    private $session;
    private $entry_handler;

    public function __construct()
    {
        MainHelper::$plugin = $this;
        MainHelper::$plugin_name =
            DuneSystem::$properties['plugin_name'];

        Lang::static_init();

        $this->session = new Session();
        $this->entry_handler = new MainEntryHandler($this->session);
        $this->tv = new MainTv();
        MainHelper::$tv = $this->tv;

        $this->add_screen(new GlobalTvTvSectionListScreen($this->tv,
                FolderViews::GET_TV_SECTION_LIST_FOLDER_VIEWS()));
        $this->add_screen(new GlobalTvTvCategoryListScreen($this->tv,
                FolderViews::GET_TV_CATEGORY_LIST_FOLDER_VIEWS()));
        $this->add_screen(new GlobalTvTvChannelListScreen($this->tv,
                FolderViews::GET_TV_CHANNEL_LIST_FOLDER_VIEWS()));
        $this->add_screen(new GlobalTvTvFavoritesScreen($this->tv,
                FolderViews::GET_TV_CHANNEL_LIST_FOLDER_VIEWS()));

        ///////////////////////////////////////////////////////////////////

        MainHelper::initialize();

        if (HD::with_list_config())
            MainHelper::$list_helper = new ListHelper();

        MainHelper::$entry_handler = $this->entry_handler;
        MainHelper::$cache = new Cache();

        if (HD::with_rows_api())
        {
            EpfsHandler::initialize();

            MainHelper::$tv_rows_screen = new TvRowsScreen($this->tv, $this->session);
            MainHelper::$dummy_epf_screen = new DummyEpfScreen();

            $this->add_screen(MainHelper::$tv_rows_screen);
            $this->add_screen(MainHelper::$dummy_epf_screen);
        }
    }

    public function get_tv()
    {
    	return $this->tv;
    }

    public function get_tv_info($media_url_str, &$plugin_cookies)
    {
        if (is_null($this->tv))
            throw new Exception('TV is not supported');

        $media_url = MediaURL::decode($media_url_str);
        $ret = $this->tv->get_tv_info($media_url, $plugin_cookies);
        return $ret;
    }
}

?>
